# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

import re

from cmf.monitor.json_metrics_utils import get_beans

class JsonParserJmx(object):
  def __init__(self, json_dict, key):
    """
    May raise ParseError if the json structure is not what we expect.
    """
    beans = BEANS[key]
    json_beans = get_beans(json_dict, map(lambda x: x.regex, beans))
    for bean in beans:
      setattr(self, bean.alias, json_beans.get(bean.regex))

class Bean:
  """
  Holds metadata used to represent a bean from which we collect metrics.
  """
  def __init__(self, regex, alias):
      # A regular expression used to identify the bean.
      self.regex = regex
      # The name we'll use internally for the bean.
      self.alias = alias

_JAVA_MEMORY_BEAN = Bean(re.compile('java.lang:type=Memory$'), 'java_memory')
_EVENT_STATS_BEAN = \
    Bean(re.compile('com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean'),
         'event_stats')

def _make_jvm_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=JvmMetrics' % name), 'jvm')

def _make_ugi_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=UgiMetrics' % name), 'ugi')

def _make_rpc_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=RpcActivityForPort\d+' 
                           % name),
                'rpc')

def _make_detailed_rpc_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=RpcDetailedActivityForPort\d+' 
                           % name),
                'detailed_rpc')

def _make_metrics_control_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=MetricsSystem,sub=Control' 
                           % name),
                'metrics_control')

def _make_metrics_stats_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=MetricsSystem,sub=Stats' 
                           % name),
                'metrics_stats')

BEANS = dict(
    NAMENODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('NameNode'),
        _make_rpc_metrics_bean('NameNode'),
        _make_detailed_rpc_metrics_bean('NameNode'),
        _make_ugi_metrics_bean('NameNode'),
        _make_metrics_control_metrics_bean('NameNode'),
        _make_metrics_stats_metrics_bean('NameNode'),
        Bean(re.compile('Hadoop:service=NameNode,name=NameNodeInfo'),
             'namenode_info'),
        Bean(re.compile('Hadoop:service=NameNode,name=NameNodeActivity'),
             'namenode_activity'),
        Bean(re.compile('Hadoop:service=NameNode,name=FSNamesystem$'),
             'fsnamesystem'),
        Bean(re.compile('Hadoop:service=NameNode,name=FSNamesystemState'), 
             'fsnamesystem_state'),
        Bean(re.compile('Hadoop:service=NameNode,name=IPCLoggerChannel'), 
             'ipc_logger_channel'),
        ],
    SECONDARYNAMENODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('SecondaryNameNode'),
        _make_rpc_metrics_bean('SecondaryNameNode'),
        _make_detailed_rpc_metrics_bean('SecondaryNameNode'),
        _make_ugi_metrics_bean('SecondaryNameNode'),
        _make_metrics_control_metrics_bean('SecondaryNameNode'),
        _make_metrics_stats_metrics_bean('SecondaryNameNode')
        ],
    DATANODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('DataNode'),
        _make_rpc_metrics_bean('DataNode'),
        _make_detailed_rpc_metrics_bean('DataNode'),
        _make_ugi_metrics_bean('DataNode'),
        _make_metrics_control_metrics_bean('DataNode'),
        _make_metrics_stats_metrics_bean('DataNode'),
        Bean(re.compile('Hadoop:service=DataNode,name=DataNodeActivity'), 
             'datanode_activity'),
        Bean(re.compile('Hadoop:service=DataNode,name=DataNodeInfo'), 
             'datanode_info'),
        Bean(re.compile('Hadoop:service=DataNode,name=FSDatasetState'), 
             'fs_dataset_state'),
        ],
    JOURNALNODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('JournalNode'),
        _make_rpc_metrics_bean('JournalNode'),
        _make_detailed_rpc_metrics_bean('JournalNode'),
        _make_ugi_metrics_bean('JournalNode'),
        _make_metrics_control_metrics_bean('JournalNode'),
        _make_metrics_stats_metrics_bean('JournalNode'),
        Bean(re.compile('Hadoop:service=JournalNode,name=Journal-.+'), 
             'journal')
        ],
    SERVICEMONITOR=[
        _EVENT_STATS_BEAN,
        Bean(re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore"), 
             'role_stage')             
        ],
    HOSTMONITOR=[
        _EVENT_STATS_BEAN,
        Bean(re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore"),
             'host_stage')
        ],
    ACTIVITYMONITOR=[
        Bean(re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree"), 
             'activity_tree_pipeline'),
        Bean(re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring"), 
             'activity_monitor_pipeline'),
        Bean(re.compile("com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService"), 
             'activity_monitor_purger')
        ],
    EVENTSERVER=[
        Bean(re.compile('com.cloudera.cmf.eventcatcher.server:type=EventCatcherServiceMXBean'), 
             'event_server'),
        Bean(re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer"), 
             'event_pipeline')
        ],
    NAVIGATOR=[
        Bean(re.compile("com.cloudera.navigator.ipc:type=NavigatorIPC,name=read-request"),
             'read_requests'),
        Bean(re.compile("com.cloudera.navigator.ipc:type=NavigatorIPC,name=insert-request"),
             'insert_requests'),
        Bean(re.compile("com.cloudera.navigator.ipc:type=NavigatorIPC,name=insertion-rate"),
             'insertion_rate'),
        Bean(re.compile("com.cloudera.navigator.ipc:type=NavigatorIPC,name=active-read-requests"),
             'active_read_requests'),
        Bean(re.compile("com.cloudera.navigator.ipc:type=NavigatorIPC,name=active-write-requests"),
             'active_write_requests')
        ],
    )

