// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
  "cloudera/Util",
  "cloudera/common/I18n",
  "underscore",
  "knockout",
  "komapping"
], function (Util, I18n, _, ko, komapping) {
  var FacetEntry = function(labelId, key) {
    this.label = I18n.t(labelId);
    this.value = key;
  };

  var ActionableEntry = function(key, values) {
    this.key = key;
    this.values = ko.observableArray(values);
  };

  /**
   * options {
   *   updateUri: (required) "the URL to fetch the data"
   *   container: (required) "selector of the container DOM object"
   *   mgmtClusterName : (required) "the name of the mgmt cluster"
   *   groupBy:   (optional) "which field to group on"
   * }
   */
  var Actionables = function(options) {
    var self = this, $container = $(options.container);

    self.showErrors = ko.observable(false);
    self.showInfos = ko.observable(false);
    self.showWarnings = ko.observable(false);

    self.toggleErrors = function() {
      self.showErrors(!self.showErrors());
    };

    self.toggleInfos= function() {
      self.showInfos(!self.showInfos());
    };

    self.toggleWarnings = function() {
      self.showWarnings(!self.showWarnings());
    };

    self.groupBy = ko.observable(options.groupBy);
    self.loaded = ko.observable(false);

    self.data = ko.observableArray();
    self.isNoIssues = ko.computed(function() {
      return self.data().length === 0;
    });
    self.allFacets = ko.observableArray();

    self.hasEntry = function(value) {
      var result = false, groupBy = self.groupBy();
      _.each(self.data(), function(actionable) {
        if (!result && actionable.metadata[groupBy] === value) {
          result = true;
        }
      });
      return result;
    };

    self.getLevelCount = function(level) {
      return _.filter(self.data(), function(actionable) {
        return actionable.level === level;
      }).length;
    };

    self.filteredData = ko.computed(function() {
      return _.chain(self.data())
      .filter(function(actionable) {
        return (self.showErrors() && actionable.level === "ERROR") || actionable.level !== "ERROR";
      }).filter(function(actionable) {
        return (self.showWarnings() && actionable.level === "WARN") || actionable.level !== "WARN";
      }).filter(function(actionable) {
        return (self.showInfos() && actionable.level === "INFO") || actionable.level !== "INFO";
      }).value();
    });

    /*
     * Function to sort ActionableEntry objects.  Sorts alphabetically
     * by key, but (kind of a hack) special cases any key named 
     * options.mgmtClusterName to be last.  This effectively
     * always puts the MGMT cluster last when we groupBy
     * cluster.
     */
    var actionableEntrySort = function(a, b) {
      if (a.key === options.mgmtClusterName) {
        return 1;
      } else if (b.key === options.mgmtClusterName) {
        return -1;
      } else {
        return a.key.localeCompare(b.key);
      }
    };

    self.actionablesBySelectedGroup = ko.computed(function() {
      var groupBy = self.groupBy();

      var actionables = _.chain(self.filteredData())
      .groupBy(function(actionable) {
        return actionable.metadata[groupBy];
      }).map(function(v, k) {
        var key = k;
        if (k === 'null') {
          if (groupBy === 'clusterDisplayName') {
            key = options.mgmtClusterName;
          } else {
            key = I18n.t("ui.other");
          }
        }
        return new ActionableEntry(key, v);
      }).value();

      return actionables.sort(actionableEntrySort);
    });

    var initData = function(response) {
      self.loaded(true);

      self.data.removeAll();
      _.each(response, function(actionable) {
        self.data.push(actionable);
      });

      self.showErrors(self.getLevelCount("ERROR") > 0);
      self.showWarnings(!self.showErrors() && self.getLevelCount("WARN") > 0);
      self.showInfos(!self.showErrors() && !self.showWarnings() && self.getLevelCount("INFO") > 0);

      var key2LabelId = {}, result = ko.observableArray();
      _.each(self.data(), function(actionable) {
        _.each(actionable.metadata, function(v, k) {
          key2LabelId[k] = "ui.facet." + k;
        });
      });
      self.allFacets.removeAll();
      _.each(key2LabelId, function(labelId, key) {
        self.allFacets.push(new FacetEntry(labelId, key));
      });
      self.groupBy(options.groupBy);
    };

    self.isNoResults = ko.computed(function() {
      return self.actionablesBySelectedGroup().length === 0;
    });

    self.hiddenKeys = ko.observableArray();
    self.allVisibleKeys = ko.computed(function() {
      return _.pluck(self.actionablesBySelectedGroup(), "key");
    });

    self.isCollapsed = function(key) {
      var isCollapsed = _.indexOf(self.hiddenKeys(), key) !== -1;
      return isCollapsed;
    };

    /*
     * Expands all Configuration Issues subsections.
     */
    self.expandAll = function() {
      self.hiddenKeys.removeAll();
    };

    /*
     * Collapses all Configuration Issues subsections.
     */
    self.collapseAll = function() {
      self.hiddenKeys.removeAll();
      _.each(self.allVisibleKeys(), function(key) {
        self.hiddenKeys.push(key);
      });
    };

    /*
     * Returns true if any subsections are open. In this
     * case we should show an "Expand All" link.
     *
     * Returns false if any subsections are closed.  In this
     * case we should show a "Collapse All" link.
     */
    self.showExpandAll = ko.computed(function() {
      return self.hiddenKeys().length === self.allVisibleKeys().length;
    });

    self.toggle = function(key) {
      if (self.hiddenKeys.remove(key).length === 0) {
        self.hiddenKeys.push(key);
      }
    };

    var updateData = function(response) {
      self.data.removeAll();
      _.each(response, function(actionable) {
        self.data.push(actionable);
      });
    };

    var handleError = function(response) {
      Util.filterError(response.responseText);
    };

    var loadData = function(now, currentMode) {
      $.post(options.updateUri, 'json')
        .success(updateData)
        .error(handleError);
    };

    $.post(options.updateUri, 'json')
      .success(initData)
      .error(handleError);


    var handle = $.subscribe("markerDateChanged", loadData);
    self.subscriptionHandles = [handle];
    self.unsubscribe = function() {
      Util.unsubscribe(self);
    };

    // Add bootstrap tooltips to filters.
    $(options.container).find(".RegionTitle button.showTooltip").tooltip();
  };

  return Actionables;
});
